import { AggregatorClient } from '@cetusprotocol/aggregator-sdk';
import { Transaction } from '@mysten/sui/transactions';
import { SuiClient } from '@mysten/sui/client';
import { v4 as uuidv4 } from 'uuid';
import { flatten } from 'lodash';
import { handleFormatValue, mappingSuiToken, MOVE_DEX_INFO, retryOperation, } from '../utils.js';
import { getCoinPTB } from '../utils.js';
import { MVM_RPC_URL } from '../../../utils/mvm.js';
import { ChainId } from '../../../types/base.js';
const client = new SuiClient({
    url: MVM_RPC_URL,
});
const cetusClient = new AggregatorClient({
    client: client,
});
export const getRoutesCetus = async (params) => {
    try {
        const router = await retryOperation(async () => {
            return await cetusClient
                .findRouters({
                from: params.fromToken,
                target: params.toToken,
                amount: BigInt(params?.fromAmount || 0),
                byAmountIn: true, // true means fix input amount, false means fix output amount
            })
                .catch((e) => {
                console.error('>>> e', e);
                throw new Error('Not found any routes from Cetus');
            });
        });
        if (params.isBuildTx && router) {
            try {
                const txb = new Transaction();
                txb.setSender(params.senderAddress || '');
                const inputCoin = await getCoinPTB(params.senderAddress || '', params.fromToken, BigInt(params?.fromAmount || 0), txb, client);
                const targetCoin = await cetusClient.routerSwap({
                    routers: router.routes,
                    byAmountIn: true,
                    txb: txb,
                    inputCoin,
                    slippage: Number(params.slippage || 0.01),
                });
                cetusClient.transferOrDestoryCoin(txb, targetCoin, params.toToken, cetusClient.publishedAtV2());
                return txb;
            }
            catch (e) {
                console.error('>>> e', e);
                throw new Error('Not build tx from Cetus');
            }
        }
        return router;
    }
    catch (e) {
        console.error(e);
    }
};
export const formatRoutesCetus = (cetusRoutes, params) => {
    const cetusInputAmount = BigInt(cetusRoutes?.value?.amountIn || 0);
    const cetusOutputAmount = BigInt(cetusRoutes?.value?.amountOut || 0);
    const cetusFromAmountUSD = handleFormatValue(params?.fromToken?.priceUSD, params?.fromToken?.decimals, params?.toToken?.priceUSD, params?.toToken?.decimals, cetusInputAmount, cetusOutputAmount).fromValue;
    const cetusToAmountUSD = handleFormatValue(params?.fromToken?.priceUSD, params?.fromToken?.decimals, params?.toToken?.priceUSD, params?.toToken?.decimals, cetusInputAmount, cetusOutputAmount).toValue;
    const formatListCetusRoute = cetusRoutes?.value?.routes?.map((item) => item.path);
    const cetusPriceImpact = (cetusToAmountUSD - cetusFromAmountUSD) / cetusFromAmountUSD;
    const formatCetusRouteSteps = flatten(formatListCetusRoute)?.map((node) => {
        return {
            id: uuidv4(),
            type: 'swap',
            action: {
                fromChainId: ChainId.MOVE,
                fromAmount: node.amountIn,
                fromToken: params.tokens?.find((t) => {
                    return mappingSuiToken(node.from) === t.address;
                }),
                toChainId: ChainId.MOVE,
                toToken: params.tokens?.find((t) => {
                    return mappingSuiToken(node.target) === t.address;
                }),
                slippage: params.options?.slippage,
                fromAddress: params.fromAddress,
            },
            estimate: {
                tool: node.provider || 'cetus pool',
                fromAmount: node.amountIn,
                toAmount: node.amountOut,
                toAmountMin: node.amountOut,
                approvalAddress: '',
                executionDuration: 1,
                feeCosts: [],
            },
            tool: node.provider || 'cetus pool',
            toolDetails: {
                key: node.provider || 'cetus pool',
                name: MOVE_DEX_INFO[node.provider]?.name ||
                    node.provider ||
                    'cetus pool',
                logoURI: MOVE_DEX_INFO[node.provider]?.image ||
                    'https://cryptologos.cc/logos/sui-sui-logo.svg?v=035',
            },
        };
    });
    return {
        id: uuidv4(),
        fromToken: params.fromToken,
        toToken: params.toToken,
        fromAmount: cetusInputAmount,
        toAmount: cetusOutputAmount,
        fromAmountUSD: cetusFromAmountUSD,
        toAmountUSD: cetusToAmountUSD,
        fromChainId: ChainId.MOVE,
        toChainId: ChainId.MOVE,
        toAmountMin: cetusOutputAmount,
        gasCostUSD: 0,
        steps: [
            {
                type: 'cetus',
                id: uuidv4(),
                tool: 'cetus',
                toolDetails: {
                    key: 'cetus',
                    name: 'Cetus',
                    logoURI: 'https://s2.coinmarketcap.com/static/img/coins/64x64/25114.png',
                },
                action: {
                    fromToken: params.fromToken,
                    fromAmount: cetusInputAmount,
                    toToken: params.toToken,
                    fromChainId: ChainId.MOVE,
                    toChainId: ChainId.MOVE,
                    slippage: params.options?.slippage,
                    fromAddress: params.fromAddress,
                    toAddress: params.toAddress || params.fromAddress,
                },
                estimate: {
                    tool: 'cetus',
                    approvalAddress: '',
                    toAmountMin: cetusOutputAmount,
                    toAmount: cetusOutputAmount,
                    fromAmount: cetusInputAmount,
                    feeCosts: [],
                    // gasCosts: [],
                    executionDuration: 1,
                    fromAmountUSD: cetusFromAmountUSD,
                    toAmountUSD: cetusToAmountUSD,
                },
                includedSteps: formatCetusRouteSteps,
                integrator: 'cetus',
                execution: {
                    status: 'PENDING',
                    process: [],
                },
            },
        ],
        path: [],
        insurance: {},
        priceImpact: cetusPriceImpact || 0,
    };
};
//# sourceMappingURL=index.js.map