import axios from 'axios';
import { v4 as uuidv4 } from 'uuid';
import { flatten } from 'lodash';
import { handleFormatValue, mappingSuiToken, MOVE_DEX_INFO } from '../utils.js';
import { ChainId } from '../../../types/base.js';
import { Transaction } from '@mysten/sui/transactions';
import { getCoinPTB, retryOperation } from '../utils.js';
import { MVM_RPC_URL } from '../../../utils/mvm.js';
import { SuiClient } from '@mysten/sui/client';
import { buildSwapPTBFromQuote } from 'navi-sdk/dist/libs/Aggregator';
const handleGetRoutesNavi = async (data) => {
    try {
        return await axios
            .get(`https://api.getnimbus.io/onchain/routes-swap?chain=SUI&type=NAVI&tokenIn=${data.tokenIn}&tokenOut=${data.tokenOut}&amountIn=${data.amountIn}`)
            .then((res) => res?.data?.data);
    }
    catch (e) {
        console.error(e);
    }
};
const client = new SuiClient({
    url: MVM_RPC_URL,
});
export const getRoutesNavi = async (params) => {
    try {
        const router = await retryOperation(async () => {
            return await handleGetRoutesNavi({
                tokenIn: params.fromToken,
                tokenOut: params.toToken,
                amountIn: BigInt(params?.fromAmount || 0),
            }).catch((e) => {
                console.error('>>> e', e);
                throw new Error('Not found any routes from Navi');
            });
        });
        if (params.isBuildTx && router) {
            try {
                const txb = new Transaction();
                txb.setSender(params.senderAddress || '');
                const coinIn = await getCoinPTB(params.senderAddress || '', params.fromToken, BigInt(params?.fromAmount || 0), txb, client);
                const minAmountOut = (Number(params.fromAmount) / 10 ** (params?.fromTokenDecimals || 9)) *
                    (1 - Number(params.slippage || 0.01));
                const targetCoin = await buildSwapPTBFromQuote(params.senderAddress || '', txb, minAmountOut, coinIn, {
                    ...router,
                    from: params.fromToken || '',
                    target: params.toToken || '',
                });
                txb.transferObjects([targetCoin], params.senderAddress || '');
                return txb;
            }
            catch (e) {
                console.error('>>> e', e);
                throw new Error('Not build tx from Navi');
            }
        }
        return router;
    }
    catch (e) {
        console.error(e);
    }
};
export const formatRoutesNavi = (naviRoutes, params) => {
    const naviInputAmount = naviRoutes?.value?.amount_in;
    const naviOutputAmount = naviRoutes?.value?.amount_out;
    const naviFromAmountUSD = handleFormatValue(params?.fromToken?.priceUSD, params?.fromToken?.decimals, params?.toToken?.priceUSD, params?.toToken?.decimals, naviInputAmount, naviOutputAmount).fromValue;
    const naviToAmountUSD = handleFormatValue(params?.fromToken?.priceUSD, params?.fromToken?.decimals, params?.toToken?.priceUSD, params?.toToken?.decimals, naviInputAmount, naviOutputAmount).toValue;
    const formatListNaviRoute = naviRoutes?.value?.routes?.map((item) => item.path);
    const naviPriceImpact = (naviToAmountUSD - naviFromAmountUSD) / naviFromAmountUSD;
    const formatNaviRouteSteps = flatten(formatListNaviRoute)?.map((node) => {
        return {
            id: uuidv4(),
            type: 'swap',
            action: {
                fromChainId: ChainId.MOVE,
                fromAmount: node.amount_in,
                fromToken: params.tokens?.find((t) => {
                    return mappingSuiToken(node.from) === t.address;
                }),
                toChainId: ChainId.MOVE,
                toToken: params.tokens?.find((t) => {
                    return mappingSuiToken(node.target) === t.address;
                }),
                slippage: params.options?.slippage,
                fromAddress: params.fromAddress,
            },
            estimate: {
                tool: node.provider || 'navi pool',
                fromAmount: node.amount_in,
                toAmount: node.amount_out,
                toAmountMin: node.amount_out,
                approvalAddress: '',
                executionDuration: 1,
                feeCosts: [],
            },
            tool: node.provider || 'navi pool',
            toolDetails: {
                key: node.provider || 'navi pool',
                name: MOVE_DEX_INFO[node.provider]?.name ||
                    node.provider ||
                    'navi pool',
                logoURI: MOVE_DEX_INFO[node.provider]?.image ||
                    'https://cryptologos.cc/logos/sui-sui-logo.svg?v=035',
            },
        };
    });
    return {
        id: uuidv4(),
        fromToken: params.fromToken,
        toToken: params.toToken,
        fromAmount: naviInputAmount,
        toAmount: naviOutputAmount,
        fromAmountUSD: naviFromAmountUSD,
        toAmountUSD: naviToAmountUSD,
        fromChainId: ChainId.MOVE,
        toChainId: ChainId.MOVE,
        toAmountMin: naviOutputAmount,
        gasCostUSD: 0,
        steps: [
            {
                type: 'navi',
                id: uuidv4(),
                tool: 'navi',
                toolDetails: {
                    key: 'navi',
                    name: 'NAVI',
                    logoURI: 'https://s2.coinmarketcap.com/static/img/coins/64x64/29296.png',
                },
                action: {
                    fromToken: params.fromToken,
                    fromAmount: naviInputAmount,
                    toToken: params.toToken,
                    fromChainId: ChainId.MOVE,
                    toChainId: ChainId.MOVE,
                    slippage: params.options?.slippage,
                    fromAddress: params.fromAddress,
                    toAddress: params.toAddress || params.fromAddress,
                },
                estimate: {
                    tool: 'navi',
                    approvalAddress: '',
                    toAmountMin: naviOutputAmount,
                    toAmount: naviOutputAmount,
                    fromAmount: naviInputAmount,
                    feeCosts: [],
                    // gasCosts: [],
                    executionDuration: 1,
                    fromAmountUSD: naviFromAmountUSD,
                    toAmountUSD: naviToAmountUSD,
                },
                includedSteps: formatNaviRouteSteps,
                integrator: 'navi',
                execution: {
                    status: 'PENDING',
                    process: [],
                },
            },
        ],
        path: [],
        insurance: {},
        priceImpact: naviPriceImpact || 0,
    };
};
//# sourceMappingURL=index.js.map