import { ECLIPSE_RPC_URL } from '../../../utils/eclipse.js';
import { Connection, PublicKey } from '@solana/web3.js';
import { BN } from '@project-serum/anchor';
import {
  getMarketAddress,
  type IWallet,
  Market,
  Pair,
  Network,
} from '@invariant-labs/sdk-eclipse';
import {
  fromFee,
  simulateSwap,
  SimulationStatus,
} from '@invariant-labs/sdk-eclipse/lib/utils.js';
import type { Tick, Tickmap } from '@invariant-labs/sdk-eclipse/lib/market.js';
import {
  TICK_CROSSES_PER_IX,
  TICK_VIRTUAL_CROSSES_PER_IX,
} from '@invariant-labs/sdk-eclipse/lib/market.js';
import { v4 as uuidv4 } from 'uuid';

import type { RoutesInvariantParams } from './types.js';
import { handleFormatValue } from '../utils.js';
import { ChainId } from '../../../types/base.js';
import { findPairs, getAllPools } from './utils.js';

const MAX_U64 = new BN('18446744073709551615');
const WRAPPED_ETH_ADDRESS = 'So11111111111111111111111111111111111111112';
const MAX_CROSSES_IN_SINGLE_TX = 11;

const findPoolIndex = (address: PublicKey, pools: any[]) => {
  return pools.findIndex((pool) => pool.address.equals(address));
};

const fetchNearestTicksForPair = async (
  market: any,
  tokenFrom: PublicKey,
  tokenTo: PublicKey,
  allPools: any,
) => {
  enum IsXtoY {
    Up = 'up',
    Down = 'down',
  }

  try {
    const marketProgram = market;

    const pools = findPairs(tokenFrom, tokenTo, allPools);

    const response = [];

    const results = await Promise.all(
      pools.map(async (pool) => {
        const isXtoY = tokenFrom.equals(pool.tokenX);
        return marketProgram.getClosestTicks(
          new Pair(tokenFrom, tokenTo, {
            fee: pool.fee,
            tickSpacing: pool.tickSpacing,
          }),
          TICK_CROSSES_PER_IX + 3,
          undefined,
          isXtoY ? IsXtoY.Down : IsXtoY.Up,
        );
      }),
    );

    if (results.length > 0) {
      for (let i = 0; i < pools.length; i++) {
        response.push({
          index: pools[i].address.toString(),
          tickStructure: results[i],
        });
      }
    }

    return response;
  } catch (error) {
    console.error(error);
  }
};

const convertBalanceToBN = (amount: string, decimals: number): BN => {
  const balanceString = amount.split('.');
  if (balanceString.length !== 2) {
    return new BN(balanceString[0] + '0'.repeat(decimals));
  }
  if (balanceString[1].length <= decimals) {
    return new BN(
      balanceString[0] +
        balanceString[1] +
        '0'.repeat(decimals - balanceString[1].length),
    );
  }
  return new BN(0);
};

const handleSimulate = async (
  pools: any[],
  poolTicks: Map<string, Tick[]>,
  tickmaps: Map<string, Tickmap>,
  slippage: BN,
  fromToken: PublicKey,
  toToken: PublicKey,
  amount: BN,
  byAmountIn: boolean,
): Promise<{
  amountOut: BN;
  poolIndex: number;
  AmountOutWithFee: BN;
  estimatedPriceAfterSwap: BN;
  minimumReceived: BN;
  priceImpact: BN;
  error: string[];
}> => {
  const filteredPools = findPairs(fromToken, toToken, pools);

  const errorMessage: string[] = [];
  let isXtoY = false;
  let result;
  let okChanges = 0;
  let failChanges = 0;
  const initAmountOut = byAmountIn ? new BN(-1) : MAX_U64;

  let successData = {
    amountOut: initAmountOut,
    poolIndex: 0,
    AmountOutWithFee: new BN(0),
    estimatedPriceAfterSwap: new BN(0),
    minimumReceived: new BN(0),
    priceImpact: new BN(0),
  };

  let allFailedData = {
    amountOut: initAmountOut,
    poolIndex: 0,
    AmountOutWithFee: new BN(0),
    estimatedPriceAfterSwap: new BN(0),
    minimumReceived: new BN(0),
    priceImpact: new BN(0),
  };

  if (amount.eq(new BN(0))) {
    return {
      amountOut: new BN(0),
      poolIndex: 0,
      AmountOutWithFee: new BN(0),
      estimatedPriceAfterSwap: new BN(0),
      minimumReceived: new BN(0),
      priceImpact: new BN(0),
      error: errorMessage,
    };
  }

  for (const pool of filteredPools) {
    isXtoY = fromToken.equals(pool.tokenX);

    const ticks: Map<number, Tick> = new Map<number, Tick>();
    const poolTicksForAddress = poolTicks.get(pool.address.toString());

    if (poolTicksForAddress) {
      for (const tick of poolTicksForAddress) {
        ticks.set(tick.index, tick);
      }
    } else {
      errorMessage.push(
        `Ticks not available for pool ${pool.address.toString()}`,
      );
      continue;
    }

    const maxCrosses =
      pool.tokenX.toString() === WRAPPED_ETH_ADDRESS ||
      pool.tokenY.toString() === WRAPPED_ETH_ADDRESS
        ? MAX_CROSSES_IN_SINGLE_TX
        : TICK_CROSSES_PER_IX;

    try {
      const swapSimulateResult = simulateSwap({
        xToY: isXtoY,
        byAmountIn: byAmountIn,
        swapAmount: amount,
        slippage: slippage,
        pool: pool,
        ticks: ticks,
        tickmap: tickmaps.get(pool.tickmap.toString()) as Tickmap,
        maxCrosses,
        maxVirtualCrosses: TICK_VIRTUAL_CROSSES_PER_IX,
      });

      if (!byAmountIn) {
        result = swapSimulateResult.accumulatedAmountIn.add(
          swapSimulateResult.accumulatedFee,
        );
      } else {
        result = swapSimulateResult.accumulatedAmountOut;
      }

      if (
        (byAmountIn
          ? successData.amountOut.lt(result)
          : successData.amountOut.gt(result)) &&
        swapSimulateResult.status === SimulationStatus.Ok &&
        swapSimulateResult.amountPerTick.length <= TICK_CROSSES_PER_IX
      ) {
        successData = {
          amountOut: result,
          poolIndex: findPoolIndex(pool.address, pools),
          AmountOutWithFee: result.add(swapSimulateResult.accumulatedFee),
          estimatedPriceAfterSwap: swapSimulateResult.priceAfterSwap,
          minimumReceived: swapSimulateResult.minReceived,
          priceImpact: swapSimulateResult.priceImpact,
        };

        okChanges += 1;
      } else if (
        byAmountIn
          ? allFailedData.amountOut.lt(result)
          : allFailedData.amountOut.eq(MAX_U64)
            ? result
            : allFailedData.amountOut.lt(result)
      ) {
        allFailedData = {
          amountOut: result,
          poolIndex: findPoolIndex(pool.address, pools),
          AmountOutWithFee: result.add(swapSimulateResult.accumulatedFee),
          estimatedPriceAfterSwap: swapSimulateResult.priceAfterSwap,
          minimumReceived: swapSimulateResult.minReceived,
          priceImpact: swapSimulateResult.priceImpact,
        };

        failChanges += 1;
      }

      if (swapSimulateResult.status !== SimulationStatus.Ok) {
        errorMessage.push(swapSimulateResult.status);
      }
    } catch (err: any) {
      errorMessage.push(err.toString());
    }
  }

  if (okChanges === 0 && failChanges === 0) {
    return {
      amountOut: new BN(0),
      poolIndex: 0,
      AmountOutWithFee: new BN(0),
      estimatedPriceAfterSwap: new BN(0),
      minimumReceived: new BN(0),
      priceImpact: new BN(0),
      error: errorMessage,
    };
  }

  if (okChanges === 0) {
    return {
      ...allFailedData,
      error: errorMessage,
    };
  }

  return {
    ...successData,
    error: [],
  };
};

const handleFormatDataBeforeSimulate = async (
  pools: any,
  market: any,
  tokenFromPublicKey: PublicKey,
  tokenToPublicKey: PublicKey,
  byAmountIn: boolean,
  slippage: BN,
  swapAmount: BN,
) => {
  const filteredPools = findPairs(tokenFromPublicKey, tokenToPublicKey, pools);

  const allTickmaps = await Promise.all(
    filteredPools.map(async (pool) => {
      return await market
        .getTickmap(
          new Pair(pool.tokenX, pool.tokenY, {
            fee: pool.fee,
            tickSpacing: pool.tickSpacing,
          }),
        )
        .then((res: any) => {
          return {
            [pool.tickmap.toString()]: res,
          };
        });
    }),
  );

  const tickmaps = allTickmaps.reduce((acc, obj) => {
    for (const [key, value] of Object.entries(obj)) {
      acc.set(key, value);
    }
    return acc;
  }, new Map());

  const allPoolTicks = await fetchNearestTicksForPair(
    market,
    tokenFromPublicKey,
    tokenToPublicKey,
    pools,
  );

  const poolTicks =
    allPoolTicks &&
    allPoolTicks.reduce((acc: Map<string, any>, current: any) => {
      acc.set(current.index, current.tickStructure);
      return acc;
    }, new Map());

  const simulation = await handleSimulate(
    pools,
    poolTicks as any,
    tickmaps as any,
    slippage,
    tokenFromPublicKey,
    tokenToPublicKey,
    swapAmount,
    byAmountIn,
  );

  return simulation;
};

export const getRoutesInvariant = async (params: RoutesInvariantParams) => {
  try {
    const connection = new Connection(ECLIPSE_RPC_URL);

    const market = await Market.build(
      Network.MAIN,
      params.account.connector as IWallet,
      connection,
      new PublicKey(getMarketAddress(Network.MAIN)),
    );

    const tokenFromPublicKey = new PublicKey(params.fromToken);
    const tokenToPublicKey = new PublicKey(params.toToken);

    const listPools = await getAllPools(
      tokenFromPublicKey,
      tokenToPublicKey,
      market,
    );

    const slippage = fromFee(new BN(Number(params.slippage * 100)));

    const swapAmount = convertBalanceToBN(
      (
        Number(params.fromAmount) /
        10 ** Number(params.fromTokenDecimals)
      ).toString(),
      params.fromTokenDecimals,
    );

    const byAmountIn = true;

    const router = await handleFormatDataBeforeSimulate(
      listPools,
      market,
      tokenFromPublicKey,
      tokenToPublicKey,
      byAmountIn,
      slippage,
      swapAmount,
    ).catch((e) => {
      console.error('>>> e', e);
      throw new Error('Not found any routes from Invariant');
    });

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesInvariant = (invariantRoutes: any, params: any) => {
  const invariantInputAmount = params?.fromAmount;
  const invariantOutputAmount = (
    invariantRoutes as any
  )?.value?.amountOut?.toString();

  const invariantFromAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    invariantInputAmount,
    invariantOutputAmount,
  ).fromValue;

  const invariantToAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    invariantInputAmount,
    invariantOutputAmount,
  ).toValue;

  const invariantPriceImpact =
    (invariantToAmountUSD - invariantFromAmountUSD) / invariantFromAmountUSD;

  const formatInvariantRouteSteps = [
    {
      id: uuidv4(),
      type: 'swap',
      action: {
        fromChainId: ChainId.ECLIPSE,
        fromAmount: invariantInputAmount,
        fromToken: params.fromToken,
        toChainId: ChainId.ECLIPSE,
        toToken: params.toToken,
        slippage: params.options?.slippage,
        fromAddress: params.fromAddress,
      },
      estimate: {
        tool: 'invariant',
        fromAmount: invariantInputAmount,
        toAmount: invariantOutputAmount,
        toAmountMin: invariantOutputAmount,
        approvalAddress: '',
        executionDuration: 1,
        feeCosts: [],
        gasCosts: [],
      },
      tool: 'invariant',
      toolDetails: {
        key: 'invariant',
        name: 'Invariant',
        logoURI: 'https://icons.llamao.fi/icons/protocols/invariant?w=48&h=48',
      },
    },
  ];

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: invariantInputAmount,
    toAmount: invariantOutputAmount,
    fromAmountUSD: invariantFromAmountUSD,
    toAmountUSD: invariantToAmountUSD,
    fromChainId: ChainId.ECLIPSE,
    toChainId: ChainId.ECLIPSE,
    toAmountMin: invariantOutputAmount,

    gasCostUSD: 0,

    steps: [
      {
        type: 'invariant',
        id: uuidv4(),
        tool: 'invariant',
        toolDetails: {
          key: 'invariant',
          name: 'Invariant',
          logoURI:
            'https://icons.llamao.fi/icons/protocols/invariant?w=48&h=48',
        },

        action: {
          fromToken: params.fromToken,
          fromAmount: invariantInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.ECLIPSE,
          toChainId: ChainId.ECLIPSE,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'invariant',
          approvalAddress: '',
          toAmountMin: invariantOutputAmount,
          toAmount: invariantOutputAmount,
          fromAmount: invariantInputAmount,
          feeCosts: [],
          // gasCosts: [],
          executionDuration: 1,
          fromAmountUSD: invariantFromAmountUSD,
          toAmountUSD: invariantToAmountUSD,
        },

        includedSteps: formatInvariantRouteSteps,
        integrator: 'invariant',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: invariantPriceImpact || 0,
  };
};
