import { ECLIPSE_RPC_URL } from '../../../utils/eclipse.js';
import { v4 as uuidv4 } from 'uuid';
import { ChainId } from '../../../types/base.js';
import { handleFormatValue } from '../utils.js';
import { Connection, PublicKey } from '@solana/web3.js';
import { getAmm, getAmountOut, getPool } from '@lifinity/sdk-v2-eclipse';

import type { RoutesLifinityParams } from './types.js';

export const getRoutesLifinity = async (params: RoutesLifinityParams) => {
  try {
    const connection: Connection = new Connection(ECLIPSE_RPC_URL);

    const fromMint: PublicKey = new PublicKey(params.fromToken);
    const toMint: PublicKey = new PublicKey(params.toToken);

    const slippage: number = Number(params.slippage) * 100;

    const amountIn =
      Number(params.fromAmount) / 10 ** Number(params.fromTokenDecimals);

    const pool = await getPool(connection as any, fromMint, toMint);

    const ammData = await getAmm(connection as any, pool.ammPubkey);

    const router = await getAmountOut(
      connection as any,
      ammData,
      amountIn,
      fromMint,
      slippage,
    ).catch((e) => {
      console.error('>>> e', e);
      throw new Error('Not found any routes from Lifinity');
    });

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesLifinity = (lifinityRoutes: any, params: any) => {
  const lifinityInputAmount =
    (lifinityRoutes as any)?.value?.amountIn *
    10 ** Number(params?.fromToken?.decimals);

  const lifinityOutputAmount =
    (lifinityRoutes as any)?.value?.amountOut *
    10 ** Number(params?.toToken?.decimals);

  const lifinityFromAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    lifinityInputAmount,
    lifinityOutputAmount,
  ).fromValue;

  const lifinityToAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    lifinityInputAmount,
    lifinityOutputAmount,
  ).toValue;

  const lifinityPriceImpact = (lifinityRoutes as any)?.value?.priceImpact;

  const formatLifinityRouteSteps = [
    {
      id: uuidv4(),
      type: 'swap',
      action: {
        fromChainId: ChainId.ECLIPSE,
        fromAmount: lifinityInputAmount,
        fromToken: params.fromToken,
        toChainId: ChainId.ECLIPSE,
        toToken: params.toToken,
        slippage: params.options?.slippage,
        fromAddress: params.fromAddress,
      },
      estimate: {
        tool: 'lifinity',
        fromAmount: lifinityInputAmount,
        toAmount: Math.round(lifinityOutputAmount),
        toAmountMin: lifinityOutputAmount,
        approvalAddress: '',
        executionDuration: 1,
        feeCosts: [],
        gasCosts: [],
      },
      tool: 'lifinity pool',
      toolDetails: {
        key: 'lifinity pool',
        name: 'Lifinity',
        logoURI:
          'https://s2.coinmarketcap.com/static/img/coins/64x64/19842.png',
      },
    },
  ];

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: lifinityInputAmount,
    toAmount: Math.round(lifinityOutputAmount),
    fromAmountUSD: lifinityFromAmountUSD,
    toAmountUSD: lifinityToAmountUSD,
    fromChainId: ChainId.ECLIPSE,
    toChainId: ChainId.ECLIPSE,
    toAmountMin: lifinityOutputAmount,

    gasCostUSD: 0,

    steps: [
      {
        type: 'lifinity',
        id: uuidv4(),
        tool: 'lifinity',
        toolDetails: {
          key: 'lifinity',
          name: 'Lifinity',
          logoURI:
            'https://s2.coinmarketcap.com/static/img/coins/64x64/19842.png',
        },

        action: {
          fromToken: params.fromToken,
          fromAmount: lifinityInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.ECLIPSE,
          toChainId: ChainId.ECLIPSE,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'lifinity',
          approvalAddress: '',
          toAmountMin: lifinityOutputAmount,
          toAmount: Math.round(lifinityOutputAmount),
          fromAmount: lifinityInputAmount,
          feeCosts: [],
          // gasCosts: [],
          executionDuration: 1,
          fromAmountUSD: lifinityFromAmountUSD,
          toAmountUSD: lifinityToAmountUSD,
        },

        includedSteps: formatLifinityRouteSteps,
        integrator: 'lifinity',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: lifinityPriceImpact || 0,
  };
};
