import axios from 'axios';
import { API_URLS } from 'solar-sdk';
import { v4 as uuidv4 } from 'uuid';
import { flatten } from 'lodash';
import type { RoutesSolarParams } from './types.js';
import { handleFormatValue } from '../utils.js';
import { ChainId } from '../../../types/base.js';

export const handleGetRoutesSolar = async (data: any) => {
  try {
    return await axios
      .get(
        `${API_URLS.SWAP_HOST}/compute/swap-base-in?inputMint=${data.tokenIn}&outputMint=${data.tokenOut}&amount=${data.amountIn}&slippageBps=${Number(data.slippage) * 100}&txVersion=${'V0'}`,
      )
      .then((res) => {
        return res?.data;
      });
  } catch (e) {
    console.error(e);
  }
};

export const getRoutesSolar = async (params: RoutesSolarParams) => {
  try {
    const router = await handleGetRoutesSolar({
      tokenIn: params.fromToken,
      tokenOut: params.toToken,
      amountIn: BigInt(params?.fromAmount || 0),
      slippage: params.slippage,
    }).catch((e) => {
      console.error('>>> e', e);
      throw new Error('Not found any routes from Solar');
    });

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesSolar = (solarRoutes: any, params: any) => {
  const solarInputAmount = (solarRoutes as any)?.value?.data?.inputAmount;
  const solarOutputAmount = (solarRoutes as any)?.value?.data?.outputAmount;

  const solarFromAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    solarInputAmount,
    solarOutputAmount,
  ).fromValue;

  const solarToAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    Number(params?.toToken?.priceUSD),
    params?.toToken?.decimals,
    solarInputAmount,
    solarOutputAmount,
  ).toValue;

  const formatListSolarRoute = (solarRoutes as any)?.value?.data?.routePlan;

  const solarPriceImpact =
    (solarToAmountUSD - solarFromAmountUSD) / solarFromAmountUSD;

  const formatSolarRouteSteps = flatten(formatListSolarRoute)?.map(
    (node: any) => {
      return {
        id: uuidv4(),
        type: 'swap',
        action: {
          fromChainId: ChainId.ECLIPSE,
          fromAmount: solarInputAmount,
          fromToken: params.fromToken,
          toChainId: ChainId.ECLIPSE,
          toToken: params.toToken,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
        },
        estimate: {
          tool: 'solar',
          fromAmount: solarInputAmount,
          toAmount: solarOutputAmount,
          toAmountMin: solarOutputAmount,
          approvalAddress: '',
          executionDuration: 1,
          feeCosts: [],
          gasCosts: [
            {
              type: 'SEND',
              price: params.nativeToken?.priceUSD,
              estimate: Math.floor(node.feeAmount),
              limit: Math.floor(node.feeAmount),
              amount: Math.floor(node.feeAmount),
              amountUSD:
                node.feeAmount * Number(params?.nativeToken?.priceUSD || 0),
              token: params?.nativeToken,
            },
          ],
        },
        tool: 'solar pool',
        toolDetails: {
          key: 'solar pool',
          name: 'Solar Dex',
          logoURI:
            'https://assets.coingecko.com/coins/images/34598/standard/solar-dex.jpeg?1705486051',
        },
      };
    },
  );

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: solarInputAmount,
    toAmount: solarOutputAmount,
    fromAmountUSD: solarFromAmountUSD,
    toAmountUSD: solarToAmountUSD,
    fromChainId: ChainId.ECLIPSE,
    toChainId: ChainId.ECLIPSE,
    toAmountMin: solarOutputAmount,

    gasCostUSD: 0,

    steps: [
      {
        type: 'solar',
        id: uuidv4(),
        tool: 'solar',
        toolDetails: {
          key: 'solar',
          name: 'Solar Dex',
          logoURI:
            'https://assets.coingecko.com/coins/images/34598/standard/solar-dex.jpeg?1705486051',
        },

        action: {
          fromToken: params.fromToken,
          fromAmount: solarInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.ECLIPSE,
          toChainId: ChainId.ECLIPSE,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'solar',
          approvalAddress: '',
          toAmountMin: solarOutputAmount,
          toAmount: solarOutputAmount,
          fromAmount: solarInputAmount,
          feeCosts: [],
          // gasCosts: [],
          executionDuration: 1,
          fromAmountUSD: solarFromAmountUSD,
          toAmountUSD: solarToAmountUSD,
        },

        includedSteps: formatSolarRouteSteps,
        integrator: 'solar',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: solarPriceImpact || 0,
  };
};
