import { TradeBuilder, AggregatorQuoter } from '@flowx-finance/sdk';
import { v4 as uuidv4 } from 'uuid';
import { flatten } from 'lodash';
import {
  handleFormatValue,
  mappingSuiToken,
  MOVE_DEX_INFO,
  retryOperation,
} from '../utils.js';
import { ChainId } from '../../../types/base.js';
import { MVM_RPC_URL } from '../../../utils/mvm.js';
import { SuiClient } from '@mysten/sui/client';
import type { RoutesFlowXParams } from './types.js';

const quoter = new AggregatorQuoter('mainnet');

const client = new SuiClient({
  url: MVM_RPC_URL,
});

export const getRoutesFlowX = async (params: RoutesFlowXParams) => {
  try {
    const router = await retryOperation(async () => {
      return await quoter
        .getRoutes({
          tokenIn: params.fromToken,
          tokenOut: params.toToken,
          amountIn: BigInt(params?.fromAmount || 0),
        } as any)
        .catch((e) => {
          console.error('>>> e', e);
          throw new Error('Not found any routes from FlowX');
        });
    });

    if (params.isBuildTx && router) {
      const tradeBuilder = new TradeBuilder('mainnet', router.routes);

      try {
        const txb = await tradeBuilder
          .sender(params.senderAddress || '')
          .slippage(Number(params.slippage)) // settings slippage or default 1%
          .build()
          .buildTransaction({ client: client as any });

        return txb as any;
      } catch (e) {
        console.error('>>> e', e);
        throw new Error('Not build tx from FlowX');
      }
    }

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesFlowX = (flowxRoutes: any, params: any) => {
  const flowXInputAmount = BigInt((flowxRoutes as any)?.value?.amountIn || 0);
  const flowXOutputAmount = BigInt((flowxRoutes as any)?.value?.amountOut || 0);

  const flowXFromAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    params?.toToken?.priceUSD,
    params?.toToken?.decimals,
    flowXInputAmount,
    flowXOutputAmount,
  ).fromValue;

  const flowXToAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    params?.toToken?.priceUSD,
    params?.toToken?.decimals,
    flowXInputAmount,
    flowXOutputAmount,
  ).toValue;

  const formatListFlowXRoute = (flowxRoutes as any)?.value?.routes?.map(
    (item: any) => item.paths,
  );

  const flowxPriceImpact =
    (flowXToAmountUSD - flowXFromAmountUSD) / flowXFromAmountUSD;

  const formatFlowXRouteSteps = flatten(formatListFlowXRoute)?.map(
    (node: any) => {
      return {
        id: uuidv4(),
        type: 'swap',
        action: {
          fromChainId: ChainId.MOVE,
          fromAmount: node.amountIn,
          fromToken: params.tokens?.find((t: any) => {
            return mappingSuiToken(node.input.coinType) === t.address;
          }),
          toChainId: ChainId.MOVE,
          toToken: params.tokens?.find((t: any) => {
            return mappingSuiToken(node.input.coinType) === t.address;
          }),
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
        },
        estimate: {
          tool: node.constructor.name || 'flowx pool',
          fromAmount: node.amountIn,
          toAmount: node.amountOut,
          toAmountMin: node.amountOut,
          approvalAddress: '',
          executionDuration: 1,
          feeCosts: [],
        },
        tool: node.constructor.name || 'flowx pool',
        toolDetails: {
          key: node.constructor.name || 'flowx pool',
          name:
            MOVE_DEX_INFO[node.constructor.name as keyof typeof MOVE_DEX_INFO]
              ?.name ||
            node.constructor.name ||
            'flowx pool',
          logoURI:
            MOVE_DEX_INFO[node.constructor.name as keyof typeof MOVE_DEX_INFO]
              ?.image || 'https://cryptologos.cc/logos/sui-sui-logo.svg?v=035',
        },
      };
    },
  );

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: flowXInputAmount,
    toAmount: flowXOutputAmount,
    fromAmountUSD: flowXFromAmountUSD,
    toAmountUSD: flowXToAmountUSD,
    fromChainId: ChainId.MOVE,
    toChainId: ChainId.MOVE,
    toAmountMin: flowXOutputAmount,

    gasCostUSD: 0,

    steps: [
      {
        type: 'flowx',
        id: uuidv4(),
        tool: 'flowx',
        toolDetails: {
          key: 'flowx',
          name: 'FlowX',
          logoURI:
            'https://assets.coingecko.com/coins/images/33317/standard/logo_flx.png?1701435649',
        },

        action: {
          fromToken: params.fromToken,
          fromAmount: flowXInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.MOVE,
          toChainId: ChainId.MOVE,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'flowx',
          approvalAddress: '',
          toAmountMin: flowXOutputAmount,
          toAmount: flowXOutputAmount,
          fromAmount: flowXInputAmount,
          feeCosts: [],
          // gasCosts: [],
          executionDuration: 1,
          fromAmountUSD: flowXFromAmountUSD,
          toAmountUSD: flowXToAmountUSD,
        },

        includedSteps: formatFlowXRouteSteps,
        integrator: 'flowx',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: flowxPriceImpact || 0,
  };
};
