import { v4 as uuidv4 } from 'uuid';
import { handleFormatValue, retryOperation } from '../utils.js';
import { ChainId } from '../../../types/base.js';
import type { RoutesJupParams } from './types.js';
import { BigNumber } from 'bignumber.js';
import axios from 'axios';

export const getRoutesJup = async (params: RoutesJupParams) => {
  try {
    const router = await retryOperation(async () => {
      const formatFromTokenCA =
        params.fromToken === '11111111111111111111111111111111'
          ? 'So11111111111111111111111111111111111111112'
          : params.fromToken;

      const formatToTokenCA =
        params.toToken === '11111111111111111111111111111111'
          ? 'So11111111111111111111111111111111111111112'
          : params.toToken;

      const response = await axios.get(
        `https://api.jup.ag/swap/v1/quote?inputMint=${formatFromTokenCA}&outputMint=${formatToTokenCA}&amount=${params.fromAmount}&maxAccounts=64&slippageBps=${params.slippage}`,
      );

      return response?.data;
    });

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesJup = (jupRoutes: any, params: any) => {
  const jupInputAmount = new BigNumber(
    Number((jupRoutes as any)?.value?.inAmount || 0),
  );

  const jupOutputAmount = new BigNumber(
    Number((jupRoutes as any)?.value?.outAmount || 0),
  );

  const jupFromAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    params?.toToken?.priceUSD,
    params?.toToken?.decimals,
    jupInputAmount,
    jupOutputAmount,
  ).fromValue;

  const jupToAmountUSD = handleFormatValue(
    params?.fromToken?.priceUSD,
    params?.fromToken?.decimals,
    params?.toToken?.priceUSD,
    params?.toToken?.decimals,
    jupInputAmount,
    jupOutputAmount,
  ).toValue;

  const jupPriceImpact = Number((jupRoutes as any)?.value?.priceImpactPct || 0);

  const formatJupRouteSteps = [
    {
      id: uuidv4(),
      type: 'swap',
      action: {
        fromChainId: ChainId.SOL,
        fromAmount: jupInputAmount,
        fromToken: params.fromToken,
        toChainId: ChainId.SOL,
        toToken: params.toToken,
        slippage: params.options?.slippage,
        fromAddress: params.fromAddress,
      },
      estimate: {
        tool: 'jupiter',
        fromAmount: jupInputAmount,
        toAmount: jupOutputAmount,
        toAmountMin: jupOutputAmount,
        approvalAddress: '',
        executionDuration: 1,
        feeCosts: [],
        gasCosts: [],
      },
      tool: 'jupiter',
      toolDetails: {
        key: 'jupiter',
        name: 'Jupiter',
        logoURI: 'https://icons.llamao.fi/icons/protocols/jupiter?w=48&h=48',
      },
    },
  ];

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: jupInputAmount,
    toAmount: jupOutputAmount,
    fromAmountUSD: jupFromAmountUSD,
    toAmountUSD: jupToAmountUSD,
    fromChainId: ChainId.SOL,
    toChainId: ChainId.SOL,
    toAmountMin: jupOutputAmount,

    gasCostUSD: 0,

    steps: [
      {
        type: 'jupiter',
        id: uuidv4(),
        tool: 'jupiter',
        toolDetails: {
          key: 'jupiter',
          name: 'Jupiter',
          logoURI: 'https://icons.llamao.fi/icons/protocols/jupiter?w=48&h=48',
        },

        action: {
          fromToken: params.fromToken,
          fromAmount: jupInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.SOL,
          toChainId: ChainId.SOL,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'jupiter',
          approvalAddress: '',
          toAmountMin: jupOutputAmount,
          toAmount: jupOutputAmount,
          fromAmount: jupInputAmount,
          feeCosts: [],
          // gasCosts: [],
          executionDuration: 1,
          fromAmountUSD: jupFromAmountUSD,
          toAmountUSD: jupToAmountUSD,
        },

        includedSteps: formatJupRouteSteps,
        integrator: 'jupiter',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: jupPriceImpact || 0,
  };
};
