import { getBestRoute } from 'rainbow-swap-sdk';
import { v4 as uuidv4 } from 'uuid';
import { flatten } from 'lodash';
import { ChainId } from '../../../types/base.js';
import { TON_DEX_INFO } from '../utils.js';
import { Address } from '@ton/core';
import { TON_NATIVE_ADDRESS } from '../../../utils/constants.js';
import type { RoutesRainbowParams } from './types.js';

const mappingTonToken = (address: string) => {
  if (address === TON_NATIVE_ADDRESS) {
    return 'ton';
  }
  return Address.parse(address).toString({
    urlSafe: true,
  });
};

export const getRoutesRainbow = async (params: RoutesRainbowParams) => {
  try {
    const assetIn = mappingTonToken(params.fromToken);

    const assetOut = mappingTonToken(params.toToken);

    const router = await getBestRoute({
      inputAssetAmount: params.fromAmount,
      inputAssetAddress: assetIn,
      outputAssetAddress: assetOut,
      senderAddress: params.senderAddress,
    }).catch((e) => {
      console.error('>>> e', e);
      throw new Error('Not found any routes from Rainbow');
    });

    if (params.isBuildTx && router) {
      return {
        swapMessages: (router as any).swapMessages,
        inputUsd: (router as any).displayData.outputAssetUsdAmount,
      };
    }

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesRainbow = (rainbowRoutes: any, params: any) => {
  const fromTokenDecimals = params?.fromToken?.decimals || 0;
  const toTokenDecimals = params?.toToken?.decimals || 0;

  const rainbowInputAmount =
    (rainbowRoutes as any)?.value?.displayData?.inputAssetAmount *
    10 ** fromTokenDecimals;

  const rainbowOutputAmount =
    (rainbowRoutes as any)?.value?.displayData?.outputAssetAmount *
    10 ** toTokenDecimals;

  const formatListRainbowRoute = (
    rainbowRoutes as any
  )?.value?.displayData?.routes?.map((item: any) => item.routeSteps || []);

  const formatRainbowRouteSteps = (
    flatten(formatListRainbowRoute).length === 0
      ? (rainbowRoutes as any)?.value?.displayData?.routes
      : flatten(formatListRainbowRoute)
  )?.map((path: any) => {
    return {
      id: uuidv4(),
      type: 'swap',
      action: {
        fromChainId: ChainId.TON,
        fromAmount: rainbowInputAmount,
        fromToken: params.fromToken,
        toChainId: ChainId.TON,
        toToken: params.toToken,
        slippage: params.options?.slippage,
        fromAddress: params.fromAddress,
      },
      estimate: {
        tool: 'rainbow',
        fromAmount: rainbowInputAmount,
        toAmount: rainbowOutputAmount,
        toAmountMin: rainbowOutputAmount,
        approvalAddress: '',
        executionDuration: 30,
        feeCosts: [],
        gasCosts: [
          {
            type: 'SEND',
            price: params.nativeToken?.priceUSD,
            estimate: Math.floor(
              (rainbowRoutes as any)?.value?.displayData?.roughGasUsdFee *
                10 ** 9,
            ),
            limit: Math.floor(
              (rainbowRoutes as any)?.value?.displayData?.roughGasUsdFee *
                10 ** 9,
            ),
            amount: Math.floor(
              (rainbowRoutes as any)?.value?.displayData?.roughGasUsdFee *
                10 ** 9,
            ),
            amountUSD:
              (rainbowRoutes as any)?.value?.displayData?.roughGasUsdFee *
              Number(params?.nativeToken?.priceUSD || 0),
            token: params?.nativeToken,
          },
        ],
      },
      tool: path?.dex?.type,
      toolDetails: {
        key: path?.dex?.type,
        name:
          TON_DEX_INFO[path?.dex?.name as keyof typeof TON_DEX_INFO]?.name ||
          path?.dex?.name,
        logoURI:
          TON_DEX_INFO[path?.dex?.name as keyof typeof TON_DEX_INFO]?.image ||
          'https://s2.coinmarketcap.com/static/img/coins/64x64/11419.png',
      },
    };
  });

  const rainbowPriceImpact =
    ((rainbowRoutes as any).value.displayData.outputAssetUsdAmount -
      (rainbowRoutes as any).value.displayData.inputAssetUsdAmount) /
    (rainbowRoutes as any).value.displayData.inputAssetUsdAmount;

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: rainbowInputAmount,
    toAmount: rainbowOutputAmount,
    fromAmountUSD: (rainbowRoutes as any).value.displayData.inputAssetUsdAmount,
    toAmountUSD: (rainbowRoutes as any).value.displayData.outputAssetUsdAmount,
    fromChainId: ChainId.TON,
    toChainId: ChainId.TON,
    toAmountMin: Math.floor(rainbowOutputAmount * (1 + rainbowPriceImpact)),
    gasCostUSD: (rainbowRoutes as any).value.displayData.roughGasUsdFee,
    steps: [
      {
        type: 'rainbow',
        id: uuidv4(),
        tool: 'rainbow',
        toolDetails: {
          key: 'rainbow',
          name: 'Rainbow',
          logoURI:
            'https://ton.app/media/8ef27922-d2e0-447b-829d-c926b946dd51.jpg?w=256&q=75',
        },
        action: {
          fromToken: params.fromToken,
          fromAmount: rainbowInputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.TON,
          toChainId: ChainId.TON,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'rainbow',
          approvalAddress: '',
          toAmountMin: rainbowOutputAmount,
          toAmount: rainbowOutputAmount,
          fromAmount: rainbowInputAmount,
          feeCosts: [],
          gasCosts: [
            {
              type: 'SEND',
              price: params.nativeToken?.priceUSD,
              estimate: Math.floor(
                (rainbowRoutes as any).value.displayData.roughGasUsdFee *
                  10 ** 9,
              ),
              limit: Math.floor(
                (rainbowRoutes as any).value.displayData.roughGasUsdFee *
                  10 ** 9,
              ),
              amount: Math.floor(
                (rainbowRoutes as any).value.displayData.roughGasUsdFee *
                  10 ** 9,
              ),
              amountUSD:
                (rainbowRoutes as any).value.displayData.roughGasUsdFee *
                Number(params.nativeToken?.priceUSD || 0),
              token: params.nativeToken,
            },
          ],
          executionDuration: 30,
          fromAmountUSD: (rainbowRoutes as any).value.displayData
            .inputAssetUsdAmount,
          toAmountUSD: (rainbowRoutes as any).value.displayData
            .outputAssetUsdAmount,
        },
        includedSteps: formatRainbowRouteSteps,
        integrator: 'rainbow',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: rainbowPriceImpact || 0,
  };
};
