import type { ApiTokenAddress } from '@swap-coffee/sdk';
import { RoutingApi } from '@swap-coffee/sdk';
import { TON_NATIVE_ADDRESS } from '../../../utils/constants.js';
import { v4 as uuidv4 } from 'uuid';
import { ChainId } from '../../../types/base.js';
import { TON_DEX_INFO } from '../utils.js';
import { Address } from '@ton/core';
import type { RoutesSwapCoffeeParams } from './types.js';

const mappingTonToken = (address: string) => {
  if (address === TON_NATIVE_ADDRESS) {
    return 'native';
  }
  return Address.parse(address).toString({
    urlSafe: true,
  });
};

const routingApi = new RoutingApi();

export const getRoutesSwapCoffee = async (params: RoutesSwapCoffeeParams) => {
  try {
    const assetIn: ApiTokenAddress = {
      blockchain: 'ton',
      address: mappingTonToken(params.fromToken || ''),
    };

    const assetOut: ApiTokenAddress = {
      blockchain: 'ton',
      address: mappingTonToken(params.toToken || ''),
    };

    const inputAmount = Number(
      (
        Number(params.fromAmount || 0) /
        10 ** (params?.fromTokenDecimals || 9)
      ).toFixed(2),
    );

    const router = await routingApi
      .buildRoute({
        input_token: assetIn,
        output_token: assetOut,
        input_amount: inputAmount,
      })
      .catch((e) => {
        console.error('>>> e', e);
        throw new Error('Not found any routes from SwapCoffee');
      });

    if (params.isBuildTx && router) {
      const transactions = await routingApi.buildTransactionsV2({
        referral_name: 'nimbus',
        sender_address: params.senderAddress || '', // address of user's wallet
        slippage: Number(params.slippage || 0.01), // settings slippage or default 1%
        paths: router.data.paths,
      });

      return {
        swapMessages: transactions,
        inputUsd: (router as any).data.input_usd,
      };
    }

    return router;
  } catch (e) {
    console.error(e);
  }
};

export const formatRoutesSwapCoffee = (swapCoffeeRoutes: any, params: any) => {
  const inputAmount = Math.floor(
    (swapCoffeeRoutes as any).value.data.paths.reduce(
      (acc: any, cur: any) => acc + cur.swap.input_amount,
      0,
    ) *
      10 ** (swapCoffeeRoutes as any).value.data.input_token.metadata.decimals,
  );

  const outputAmount = Math.floor(
    (swapCoffeeRoutes as any).value.data.paths.reduce((acc: any, cur: any) => {
      if (cur.next) {
        return acc + cur.next[cur.next.length - 1].swap.output_amount;
      }
      return acc + cur.swap.output_amount;
    }, 0) *
      10 ** (swapCoffeeRoutes as any).value.data.output_token.metadata.decimals,
  );

  const formatSwapCoffeeRouteSteps = (
    swapCoffeeRoutes as any
  )?.value?.data?.paths?.map((path: any) => ({
    id: uuidv4(),
    type: 'swap',
    action: {
      fromChainId: ChainId.TON,
      fromAmount: Math.floor(
        path.swap.input_amount *
          10 **
            (swapCoffeeRoutes as any).value.data.input_token.metadata.decimals,
      ),
      fromToken: params.fromToken,
      toChainId: ChainId.TON,
      toToken: params.toToken,
      slippage: params.options?.slippage,
      fromAddress: params.fromAddress,
    },
    estimate: {
      tool: 'swapcoffee',
      fromAmount: Math.floor(
        path.swap.input_amount *
          10 **
            (swapCoffeeRoutes as any).value.data.input_token.metadata.decimals,
      ),
      toAmount: Math.floor(
        (path.next
          ? path.next[path.next.length - 1].swap.output_amount
          : path.swap.output_amount) *
          10 **
            (swapCoffeeRoutes as any).value.data.output_token.metadata.decimals,
      ),
      toAmountMin: Math.floor(
        (path.next
          ? path.next[path.next.length - 1].swap.output_amount
          : path.swap.output_amount) *
          10 **
            (swapCoffeeRoutes as any).value.data.output_token.metadata.decimals,
      ),
      approvalAddress: '',
      executionDuration: 30,
      feeCosts: [],
      gasCosts: [
        {
          type: 'SEND',
          price: params.nativeToken?.priceUSD,
          estimate: Math.floor(
            (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
          ),
          limit: Math.floor(
            (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
          ),
          amount: Math.floor(
            (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
          ), // TON token decimal is 9
          amountUSD:
            (swapCoffeeRoutes as any).value.data.recommended_gas *
            Number(params.nativeToken?.priceUSD || 0),
          token: params.nativeToken,
        },
      ],
    },
    tool: path.dex,
    toolDetails: {
      key: path.dex,
      name:
        TON_DEX_INFO[path.dex as keyof typeof TON_DEX_INFO].name || path.dex,
      logoURI:
        TON_DEX_INFO[path.dex as keyof typeof TON_DEX_INFO]?.image ||
        'https://s2.coinmarketcap.com/static/img/coins/64x64/11419.png',
    },
  }));

  return {
    id: uuidv4(),
    fromToken: params.fromToken,
    toToken: params.toToken,
    fromAmount: inputAmount,
    toAmount: outputAmount,
    fromAmountUSD: (swapCoffeeRoutes as any).value.data.input_usd,
    toAmountUSD: (swapCoffeeRoutes as any).value.data.output_usd,
    fromChainId: ChainId.TON,
    toChainId: ChainId.TON,
    toAmountMin: Math.floor(
      outputAmount * (1 + (swapCoffeeRoutes as any).value.data.price_impact),
    ),
    gasCostUSD:
      (swapCoffeeRoutes as any).value.data.recommended_gas *
      Number(params.nativeToken?.priceUSD || 0),
    steps: [
      {
        type: 'swapcoffee',
        id: uuidv4(),
        tool: 'swapcoffee',
        toolDetails: {
          key: 'swapcoffee',
          name: 'Swap Coffee',
          logoURI: 'https://avatars.githubusercontent.com/u/171727895',
        },
        action: {
          fromToken: params.fromToken,
          fromAmount: inputAmount,
          toToken: params.toToken,
          fromChainId: ChainId.TON,
          toChainId: ChainId.TON,
          slippage: params.options?.slippage,
          fromAddress: params.fromAddress,
          toAddress: params.toAddress || params.fromAddress,
        },
        estimate: {
          tool: 'swapcoffee',
          approvalAddress: '',
          toAmountMin: outputAmount,
          toAmount: outputAmount,
          fromAmount: inputAmount,
          feeCosts: [],
          gasCosts: [
            {
              type: 'SEND',
              price: params.nativeToken?.priceUSD,
              estimate: Math.floor(
                (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
              ),
              limit: Math.floor(
                (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
              ),
              amount: Math.floor(
                (swapCoffeeRoutes as any).value.data.recommended_gas * 10 ** 9,
              ), // can hardcode because gas cost calculated from TON
              amountUSD:
                (swapCoffeeRoutes as any).value.data.recommended_gas *
                Number(params.nativeToken?.priceUSD || 0),
              token: params.nativeToken,
            },
          ],
          executionDuration: 30,
          fromAmountUSD: (swapCoffeeRoutes as any).value.data.input_usd,
          toAmountUSD: (swapCoffeeRoutes as any).value.data.output_usd,
        },
        includedSteps: formatSwapCoffeeRouteSteps,
        integrator: 'dedux',
        execution: {
          status: 'PENDING',
          process: [],
        },
      },
    ],
    path: [],
    insurance: {},
    priceImpact: (swapCoffeeRoutes as any).value.data?.price_impact || 0,
  };
};
